==> nono-0.7.2.tar.gz: nono-0.7.2/vm/mainbus.h <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// メインバス (共通部分)
//

#pragma once

#include "device.h"
#include "bus.h"
#include "monitor.h"
#include <array>
#include <utility>

// メインバスの素質を持つクラス
class MainbusBaseDevice : public IODevice
{
	using inherited = IODevice;
 public:
	MainbusBaseDevice(int objid_);
	~MainbusBaseDevice() override;

	// メインバスの初期化
	virtual bool InitMainbus() = 0;

	// MPU からのリセット (ここ?)
	virtual void ResetByMPU();

	// addr 版 (こっちは呼ばないこと)
	busdata Read8(uint32 addr) override final;
	busdata Read16(uint32 addr) override final;
	busdata Read32(uint32 addr) override final;
	busdata Write8(uint32 addr, uint32 data) override final;
	busdata Write16(uint32 addr, uint32 data) override final;
	busdata Write32(uint32 addr, uint32 data) override final;
	busdata ReadBurst16(uint32 addr, uint32 *dst) override final;
	busdata WriteBurst16(uint32 addr, const uint32 *src) override final;

	// busaddr 版
	virtual busdata Read8(busaddr addr) = 0;
	virtual busdata Read16(busaddr addr) = 0;
	virtual busdata Read32(busaddr addr) = 0;
	virtual busdata Write8(busaddr addr, uint32 data) = 0;
	virtual busdata Write16(busaddr addr, uint32 data) = 0;
	virtual busdata Write32(busaddr addr, uint32 data) = 0;
	virtual busdata ReadBurst16(busaddr addr, uint32 *dst);
	virtual busdata WriteBurst16(busaddr addr, const uint32 *src);

	// ブートページを ROM に切り替える
	void SwitchBootPageToROM() { SwitchBootPage(true); }
	// ブートページを RAM に切り替える
	void SwitchBootPageToRAM() { SwitchBootPage(false); }

	// モニタ表示用にデバイス名を整形。
	static std::pair<std::string, TA> FormatDevName(const Device *dev);

 protected:
	// ブートページを切り替える (内部用)
	virtual void SwitchBootPage(bool isrom) = 0;
};

// メインバス (システムに1つ)
class MainbusDevice : public MainbusBaseDevice
{
	using inherited = MainbusBaseDevice;
 public:
	// アクセス状況
	static const uint8 ACC_READ		= 0x01;
	static const uint8 ACC_WRITE	= 0x02;

 public:
	MainbusDevice(int objid_);
	~MainbusDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	// ハイパーバイザ的アクセスというか、
	// 今となってはミスアライン可能なアクセスというだけかも知れない。
	busdata HVRead8(uint32 addr);
	busdata HVRead16(uint32 addr);
	busdata HVRead32(uint32 addr);
	busdata HVWrite8(uint32 addr, uint32 data);
	busdata HVWrite16(uint32 addr, uint32 data);
	busdata HVWrite32(uint32 addr, uint32 data);

	uint64 Peek32(uint32 addr);

 protected:
	DECLARE_MONITOR_CALLBACK(MonitorUpdateAccStat);

	// アクセス状況は 32bit 空間全域を 2048 * 2MB ずつ。
	// (NEWS は読み出し時にマージする)
	std::array<uint8, 2048> accstat_read {};
	std::array<uint8, 2048> accstat_write {};
	// R, W をマージした表示用(要素数は機種による)
	std::vector<uint8> accstat_rw {};
	// デバイスの存在有無。MSB 側から 1bit = 16MB のビットマップ。
	std::array<uint8, 32> accstat_avail {};
	// アクセス状況が何ビット空間分か。NEWS だけ 30
	int accstat_bitlen {};

	Monitor monitor_accstat { this };
};

// 上位8ビットがテーブルで表せる構造のメインバス (共通部分)
class Mainbus24Device : public MainbusDevice
{
	using inherited = MainbusDevice;
 public:
	Mainbus24Device();
	~Mainbus24Device() override;

	busdata Read8(busaddr addr) override;
	busdata Read16(busaddr addr) override;
	busdata Read32(busaddr addr) override;
	busdata Write8(busaddr addr, uint32 data) override;
	busdata Write16(busaddr addr, uint32 data) override;
	busdata Write32(busaddr addr, uint32 data) override;
	busdata ReadBurst16(busaddr addr, uint32 *dst) override;
	busdata WriteBurst16(busaddr addr, const uint32 *src) override;
	busdata Peek8(uint32 addr) override;
	bool Poke8(uint32 addr, uint32 data) override;

 protected:
	DECLARE_MONITOR_CALLBACK(MonitorUpdate);

	void InitAccStat();

	std::array<IODevice *, 256> devtable {};

	Monitor monitor { this };

 private:
	inline IODevice *Decoder(uint32 addr) const;
};

static inline MainbusDevice *GetMainbusDevice() {
	return Object::GetObject<MainbusDevice>(OBJ_MAINBUS);
}

==> nono-0.7.3.tar.gz: nono-0.7.3/vm/mainbus.h <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// メインバス (共通部分)
//

#pragma once

#include "device.h"
#include "bus.h"
#include "monitor.h"
#include <array>
#include <utility>

// メインバスの素質を持つクラス
class MainbusBaseDevice : public IODevice
{
	using inherited = IODevice;
 public:
	explicit MainbusBaseDevice(uint objid_);
	~MainbusBaseDevice() override;

	// メインバスの初期化
	virtual bool InitMainbus() = 0;

	// MPU からのリセット (ここ?)
	virtual void ResetByMPU();

	// addr 版 (こっちは呼ばないこと)
	busdata Read8(uint32 addr) override final;
	busdata Read16(uint32 addr) override final;
	busdata Read32(uint32 addr) override final;
	busdata Write8(uint32 addr, uint32 data) override final;
	busdata Write16(uint32 addr, uint32 data) override final;
	busdata Write32(uint32 addr, uint32 data) override final;
	busdata ReadBurst16(uint32 addr, uint32 *dst) override final;
	busdata WriteBurst16(uint32 addr, const uint32 *src) override final;

	// busaddr 版
	virtual busdata Read8(busaddr addr) = 0;
	virtual busdata Read16(busaddr addr) = 0;
	virtual busdata Read32(busaddr addr) = 0;
	virtual busdata Write8(busaddr addr, uint32 data) = 0;
	virtual busdata Write16(busaddr addr, uint32 data) = 0;
	virtual busdata Write32(busaddr addr, uint32 data) = 0;
	virtual busdata ReadBurst16(busaddr addr, uint32 *dst);
	virtual busdata WriteBurst16(busaddr addr, const uint32 *src);

	// ブートページを ROM に切り替える
	void SwitchBootPageToROM() { SwitchBootPage(true); }
	// ブートページを RAM に切り替える
	void SwitchBootPageToRAM() { SwitchBootPage(false); }

	// モニタ表示用にデバイス名を整形。
	static std::pair<std::string, TA> FormatDevName(const Device *dev);

 protected:
	// ブートページを切り替える (内部用)
	virtual void SwitchBootPage(bool isrom) = 0;
};

// 定数クラス。本当はインタフェースっぽくしたい
class AccStat
{
 public:
	// アクセス状況を記録・表示する際に切り捨てる下位ビット数
	static const uint SHIFT = 21;

	// メイン空間とバンクメモリ(1枚分)の要素数
	static const uint MAINLEN	= (1 << (32 - SHIFT));
	static const uint BANKLEN	= (1 << (24 - SHIFT));

	// アクセス状況
	static const uint8 WRITE	= 0x01;
	static const uint8 READ		= 0x02;
};

// メインバス (システムに1つ)
class MainbusDevice : public MainbusBaseDevice
{
	using inherited = MainbusBaseDevice;
	friend class WXAccStatPanel;

 public:
	explicit MainbusDevice(uint objid_);
	~MainbusDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	// ハイパーバイザ的アクセスというか、
	// 今となってはミスアライン可能なアクセスというだけかも知れない。
	busdata HVRead8(uint32 addr);
	busdata HVRead16(uint32 addr);
	busdata HVRead32(uint32 addr);
	busdata HVWrite8(uint32 addr, uint32 data);
	busdata HVWrite16(uint32 addr, uint32 data);
	busdata HVWrite32(uint32 addr, uint32 data);

	uint64 Peek32(uint32 addr);

 protected:
	DECLARE_MONITOR_CALLBACK(MonitorUpdateAccStat);

	// アクセス状況は 32bit 空間全域を 2048 * 2MB ずつ。
	// (NEWS は読み出し時にマージする)
	std::array<uint8, AccStat::MAINLEN> accstat_read {};
	std::array<uint8, AccStat::MAINLEN> accstat_write {};
	// R, W をマージした表示用(要素数は機種による)
	std::vector<uint8> accstat_rw {};
	// デバイスの存在有無。MSB 側から 1bit = 16MB のビットマップ。
	std::array<uint8, 32> accstat_avail {};
	// アクセス状況のアドレス表示オフセット。NEWS だけ 0xc0
	uint accstat_baseaddr {};
	// アクセス状況が何ビット空間分か。NEWS だけ 30
	uint accstat_bitlen {};

	// X68030 バンクメモリの表示用アクセス状況。
	// (GUI からのアクセスの便宜のため共通クラスのこっちに置く)
	alignas(8) uint8 accbank[AccStat::BANKLEN * 2] {};

	Monitor monitor_accstat { this };
};

// 上位8ビットがテーブルで表せる構造のメインバス (共通部分)
class Mainbus24Device : public MainbusDevice
{
	using inherited = MainbusDevice;
 public:
	Mainbus24Device();
	~Mainbus24Device() override;

	busdata Read8(busaddr addr) override;
	busdata Read16(busaddr addr) override;
	busdata Read32(busaddr addr) override;
	busdata Write8(busaddr addr, uint32 data) override;
	busdata Write16(busaddr addr, uint32 data) override;
	busdata Write32(busaddr addr, uint32 data) override;
	busdata ReadBurst16(busaddr addr, uint32 *dst) override;
	busdata WriteBurst16(busaddr addr, const uint32 *src) override;
	busdata Peek8(uint32 addr) override;
	bool Poke8(uint32 addr, uint32 data) override;

 protected:
	DECLARE_MONITOR_CALLBACK(MonitorUpdate);

	void InitAccStat();

	std::array<IODevice *, 256> devtable {};

	Monitor monitor { this };

 private:
	inline IODevice *Decoder(uint32 addr) const;
};

static inline MainbusDevice *GetMainbusDevice() {
	return Object::GetObject<MainbusDevice>(OBJ_MAINBUS);
}
